==> forestry_1.11.2-5.3.4.106-sources.jar: forestry/energy/render/RenderEngine.java <==
/*******************************************************************************
 * Copyright (c) 2011-2014 SirSengir.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * Various Contributors including, but not limited to:
 * SirSengir (original work), CovertJaguar, Player, Binnie, MysteriousAges
 ******************************************************************************/
package forestry.energy.render;

import javax.annotation.Nullable;

import forestry.core.blocks.BlockBase;
import forestry.core.config.Constants;
import forestry.core.render.ForestryResource;
import forestry.core.tiles.TemperatureState;
import forestry.core.tiles.TileEngine;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.World;

public class RenderEngine extends TileEntitySpecialRenderer<TileEngine> {
	private final ModelRenderer boiler;
	private final ModelRenderer trunk;
	private final ModelRenderer piston;
	private final ModelRenderer extension;

	private enum Textures {

		BASE, PISTON, EXTENSION, TRUNK_HIGHEST, TRUNK_HIGHER, TRUNK_HIGH, TRUNK_MEDIUM, TRUNK_LOW
	}

	private ResourceLocation[] textures;
	private static final float[] angleMap = new float[6];

	static {
		angleMap[EnumFacing.EAST.ordinal()] = (float) -Math.PI / 2;
		angleMap[EnumFacing.WEST.ordinal()] = (float) Math.PI / 2;
		angleMap[EnumFacing.UP.ordinal()] = 0;
		angleMap[EnumFacing.DOWN.ordinal()] = (float) Math.PI;
		angleMap[EnumFacing.SOUTH.ordinal()] = (float) Math.PI / 2;
		angleMap[EnumFacing.NORTH.ordinal()] = (float) -Math.PI / 2;
	}

	public RenderEngine(String baseTexture) {
		ModelBase model = new EngineModelBase();
		boiler = new ModelRenderer(model, 0, 0);
		boiler.func_78789_a(-8F, -8F, -8F, 16, 6, 16);
		boiler.field_78800_c = 8;
		boiler.field_78797_d = 8;
		boiler.field_78798_e = 8;

		trunk = new ModelRenderer(model, 0, 0);
		trunk.func_78789_a(-4F, -4F, -4F, 8, 12, 8);
		trunk.field_78800_c = 8F;
		trunk.field_78797_d = 8F;
		trunk.field_78798_e = 8F;

		piston = new ModelRenderer(model, 0, 0);
		piston.func_78789_a(-6F, -2, -6F, 12, 4, 12);
		piston.field_78800_c = 8F;
		piston.field_78797_d = 8F;
		piston.field_78798_e = 8F;

		extension = new ModelRenderer(model, 0, 0);
		extension.func_78789_a(-5F, -3, -5F, 10, 2, 10);
		extension.field_78800_c = 8F;
		extension.field_78797_d = 8F;
		extension.field_78798_e = 8F;

		textures = new ResourceLocation[]{
				new ForestryResource(baseTexture + "base.png"),
				new ForestryResource(baseTexture + "piston.png"),
				new ForestryResource(baseTexture + "extension.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_highest.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_higher.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_high.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_medium.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_low.png"),};
	}

	@Override
	public void func_180535_a(@Nullable TileEngine engine, double x, double y, double z, float partialTicks, int destroyStage) {
		if (engine != null) {
			World worldObj = engine.getWorldObj();
			if (worldObj.func_175667_e(engine.func_174877_v())) {
				IBlockState blockState = worldObj.func_180495_p(engine.func_174877_v());
				if (blockState.func_177230_c() instanceof BlockBase) {
					EnumFacing facing = blockState.func_177229_b(BlockBase.FACING);
					render(engine.getTemperatureState(), engine.progress, facing, x, y, z);
					return;
				}
			}
		}
		render(TemperatureState.COOL, 0.25F, EnumFacing.UP, x, y, z);
	}

	private void render(TemperatureState state, float progress, EnumFacing orientation, double x, double y, double z) {
		GlStateManager.func_179124_c(1, 1, 1);

		GlStateManager.func_179094_E();
		GlStateManager.func_179109_b((float) x, (float) y, (float) z);

		float step;

		if (progress > 0.5) {
			step = 5.99F - (progress - 0.5F) * 2F * 5.99F;
		} else {
			step = progress * 2F * 5.99F;
		}

		float tfactor = step / 16;

		float[] angle = {0, 0, 0};
		float[] translate = {orientation.func_82601_c(), orientation.func_96559_d(), orientation.func_82599_e()};

		switch (orientation) {
			case EAST:
			case WEST:
			case DOWN:
				angle[2] = angleMap[orientation.ordinal()];
				break;
			case SOUTH:
			case NORTH:
			default:
				angle[0] = angleMap[orientation.ordinal()];
				break;
		}

		boiler.field_78795_f = angle[0];
		boiler.field_78796_g = angle[1];
		boiler.field_78808_h = angle[2];

		trunk.field_78795_f = angle[0];
		trunk.field_78796_g = angle[1];
		trunk.field_78808_h = angle[2];

		piston.field_78795_f = angle[0];
		piston.field_78796_g = angle[1];
		piston.field_78808_h = angle[2];

		extension.field_78795_f = angle[0];
		extension.field_78796_g = angle[1];
		extension.field_78808_h = angle[2];

		float factor = (float) (1.0 / 16.0);

		TextureManager textureManager = Minecraft.func_71410_x().func_110434_K();
		textureManager.func_110577_a(textures[Textures.BASE.ordinal()]);
		boiler.func_78785_a(factor);

		textureManager.func_110577_a(textures[Textures.PISTON.ordinal()]);
		GlStateManager.func_179109_b(translate[0] * tfactor, translate[1] * tfactor, translate[2] * tfactor);
		piston.func_78785_a(factor);
		GlStateManager.func_179109_b(-translate[0] * tfactor, -translate[1] * tfactor, -translate[2] * tfactor);

		ResourceLocation texture;

		switch (state) {
			case OVERHEATING:
				texture = textures[Textures.TRUNK_HIGHEST.ordinal()];
				break;
			case RUNNING_HOT:
				texture = textures[Textures.TRUNK_HIGHER.ordinal()];
				break;
			case OPERATING_TEMPERATURE:
				texture = textures[Textures.TRUNK_HIGH.ordinal()];
				break;
			case WARMED_UP:
				texture = textures[Textures.TRUNK_MEDIUM.ordinal()];
				break;
			case COOL:
			default:
				texture = textures[Textures.TRUNK_LOW.ordinal()];
				break;

		}
		textureManager.func_110577_a(texture);
		trunk.func_78785_a(factor);

		textureManager.func_110577_a(textures[Textures.EXTENSION.ordinal()]);
		float chamberf = 2F / 16F;

		if (step > 0) {
			for (int i = 0; i <= step + 2; i += 2) {
				extension.func_78785_a(factor);
				GlStateManager.func_179109_b(translate[0] * chamberf, translate[1] * chamberf, translate[2] * chamberf);
			}
		}

		GlStateManager.func_179121_F();
	}

	private static class EngineModelBase extends ModelBase {
	}
}

==> forestry_1.12.2-5.8.1.340-sources.jar: forestry/energy/render/RenderEngine.java <==
/*******************************************************************************
 * Copyright (c) 2011-2014 SirSengir.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * Various Contributors including, but not limited to:
 * SirSengir (original work), CovertJaguar, Player, Binnie, MysteriousAges
 ******************************************************************************/
package forestry.energy.render;

import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.World;

import forestry.core.blocks.BlockBase;
import forestry.core.config.Constants;
import forestry.core.render.ForestryResource;
import forestry.core.tiles.TemperatureState;
import forestry.core.tiles.TileEngine;

public class RenderEngine extends TileEntitySpecialRenderer<TileEngine> {
	private final ModelRenderer boiler;
	private final ModelRenderer trunk;
	private final ModelRenderer piston;
	private final ModelRenderer extension;

	private enum Textures {

		BASE, PISTON, EXTENSION, TRUNK_HIGHEST, TRUNK_HIGHER, TRUNK_HIGH, TRUNK_MEDIUM, TRUNK_LOW
	}

	private ResourceLocation[] textures;
	private static final float[] angleMap = new float[6];

	static {
		angleMap[EnumFacing.EAST.ordinal()] = (float) -Math.PI / 2;
		angleMap[EnumFacing.WEST.ordinal()] = (float) Math.PI / 2;
		angleMap[EnumFacing.UP.ordinal()] = 0;
		angleMap[EnumFacing.DOWN.ordinal()] = (float) Math.PI;
		angleMap[EnumFacing.SOUTH.ordinal()] = (float) Math.PI / 2;
		angleMap[EnumFacing.NORTH.ordinal()] = (float) -Math.PI / 2;
	}

	public RenderEngine(String baseTexture) {
		ModelBase model = new EngineModelBase();
		boiler = new ModelRenderer(model, 0, 0);
		boiler.func_78789_a(-8F, -8F, -8F, 16, 6, 16);
		boiler.field_78800_c = 8;
		boiler.field_78797_d = 8;
		boiler.field_78798_e = 8;

		trunk = new ModelRenderer(model, 0, 0);
		trunk.func_78789_a(-4F, -4F, -4F, 8, 12, 8);
		trunk.field_78800_c = 8F;
		trunk.field_78797_d = 8F;
		trunk.field_78798_e = 8F;

		piston = new ModelRenderer(model, 0, 0);
		piston.func_78789_a(-6F, -2, -6F, 12, 4, 12);
		piston.field_78800_c = 8F;
		piston.field_78797_d = 8F;
		piston.field_78798_e = 8F;

		extension = new ModelRenderer(model, 0, 0);
		extension.func_78789_a(-5F, -3, -5F, 10, 2, 10);
		extension.field_78800_c = 8F;
		extension.field_78797_d = 8F;
		extension.field_78798_e = 8F;

		textures = new ResourceLocation[]{
				new ForestryResource(baseTexture + "base.png"),
				new ForestryResource(baseTexture + "piston.png"),
				new ForestryResource(baseTexture + "extension.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_highest.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_higher.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_high.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_medium.png"),
				new ForestryResource(Constants.TEXTURE_PATH_BLOCKS + "/engine_trunk_low.png"),};
	}
	
	@Override
	public void func_192841_a(TileEngine engine, double x, double y, double z, float partialTicks, int destroyStage, float alpha) {
		if (engine != null) {
			World worldObj = engine.getWorldObj();
			if (worldObj.func_175667_e(engine.func_174877_v())) {
				IBlockState blockState = worldObj.func_180495_p(engine.func_174877_v());
				if (blockState.func_177230_c() instanceof BlockBase) {
					EnumFacing facing = blockState.func_177229_b(BlockBase.FACING);
					render(engine.getTemperatureState(), engine.progress, facing, x, y, z);
					return;
				}
			}
		}
		render(TemperatureState.COOL, 0.25F, EnumFacing.UP, x, y, z);
	}

	private void render(TemperatureState state, float progress, EnumFacing orientation, double x, double y, double z) {
		GlStateManager.func_179124_c(1, 1, 1);

		GlStateManager.func_179094_E();
		GlStateManager.func_179109_b((float) x, (float) y, (float) z);

		float step;

		if (progress > 0.5) {
			step = 5.99F - (progress - 0.5F) * 2F * 5.99F;
		} else {
			step = progress * 2F * 5.99F;
		}

		float tfactor = step / 16;

		float[] angle = {0, 0, 0};
		float[] translate = {orientation.func_82601_c(), orientation.func_96559_d(), orientation.func_82599_e()};

		switch (orientation) {
			case EAST:
			case WEST:
			case DOWN:
				angle[2] = angleMap[orientation.ordinal()];
				break;
			case SOUTH:
			case NORTH:
			default:
				angle[0] = angleMap[orientation.ordinal()];
				break;
		}

		boiler.field_78795_f = angle[0];
		boiler.field_78796_g = angle[1];
		boiler.field_78808_h = angle[2];

		trunk.field_78795_f = angle[0];
		trunk.field_78796_g = angle[1];
		trunk.field_78808_h = angle[2];

		piston.field_78795_f = angle[0];
		piston.field_78796_g = angle[1];
		piston.field_78808_h = angle[2];

		extension.field_78795_f = angle[0];
		extension.field_78796_g = angle[1];
		extension.field_78808_h = angle[2];

		float factor = (float) (1.0 / 16.0);

		TextureManager textureManager = Minecraft.func_71410_x().func_110434_K();
		textureManager.func_110577_a(textures[Textures.BASE.ordinal()]);
		boiler.func_78785_a(factor);

		textureManager.func_110577_a(textures[Textures.PISTON.ordinal()]);
		GlStateManager.func_179109_b(translate[0] * tfactor, translate[1] * tfactor, translate[2] * tfactor);
		piston.func_78785_a(factor);
		GlStateManager.func_179109_b(-translate[0] * tfactor, -translate[1] * tfactor, -translate[2] * tfactor);

		ResourceLocation texture;

		switch (state) {
			case OVERHEATING:
				texture = textures[Textures.TRUNK_HIGHEST.ordinal()];
				break;
			case RUNNING_HOT:
				texture = textures[Textures.TRUNK_HIGHER.ordinal()];
				break;
			case OPERATING_TEMPERATURE:
				texture = textures[Textures.TRUNK_HIGH.ordinal()];
				break;
			case WARMED_UP:
				texture = textures[Textures.TRUNK_MEDIUM.ordinal()];
				break;
			case COOL:
			default:
				texture = textures[Textures.TRUNK_LOW.ordinal()];
				break;

		}
		textureManager.func_110577_a(texture);
		trunk.func_78785_a(factor);

		textureManager.func_110577_a(textures[Textures.EXTENSION.ordinal()]);
		float chamberf = 2F / 16F;

		if (step > 0) {
			for (int i = 0; i <= step + 2; i += 2) {
				extension.func_78785_a(factor);
				GlStateManager.func_179109_b(translate[0] * chamberf, translate[1] * chamberf, translate[2] * chamberf);
			}
		}

		GlStateManager.func_179121_F();
	}

	private static class EngineModelBase extends ModelBase {
	}
}
